"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeGenesisAssets = exports.createGenesisBlock = exports.createChecksum = void 0;
const tslib_1 = require("tslib");
const crypto = tslib_1.__importStar(require("crypto"));
const fs = tslib_1.__importStar(require("fs-extra"));
const path_1 = tslib_1.__importDefault(require("path"));
const process_1 = require("./process");
BigInt.prototype.toJSON = function () {
    return this.toString();
};
Buffer.prototype.toJSON = function () {
    return this.toString('hex');
};
const createChecksum = async (filePath) => {
    const fileStream = fs.createReadStream(filePath);
    const dataHash = crypto.createHash('sha256');
    const fileHash = await new Promise((resolve, reject) => {
        fileStream.on('data', (datum) => {
            dataHash.update(datum);
        });
        fileStream.on('error', error => {
            reject(error);
        });
        fileStream.on('end', () => {
            resolve(dataHash.digest());
        });
    });
    return fileHash.toString('hex');
};
exports.createChecksum = createChecksum;
const createGenesisBlock = async (network, configFilepath, outputDir, blockAtSnapshotHeight, snapshotTimeGap) => {
    const height = blockAtSnapshotHeight.header.height + 1;
    const timestamp = blockAtSnapshotHeight.header.timestamp + snapshotTimeGap;
    const previousBlockID = blockAtSnapshotHeight.header.id.toString('hex');
    const genesisBlockCreateCommand = `lisk-core genesis-block:create --network ${network} --config=${configFilepath} --output=${outputDir} --assets-file=${outputDir}/genesis_assets.json --height=${height} --previous-block-id=${previousBlockID} --timestamp=${timestamp}`;
    await (0, process_1.execAsync)(genesisBlockCreateCommand);
};
exports.createGenesisBlock = createGenesisBlock;
const writeGenesisAssets = async (genesisAssets, outputDir) => {
    if (fs.existsSync(outputDir))
        fs.rmdirSync(outputDir, { recursive: true });
    fs.mkdirSync(outputDir, { recursive: true });
    const genesisAssetsJsonFilepath = path_1.default.resolve(outputDir, 'genesis_assets.json');
    fs.writeFileSync(genesisAssetsJsonFilepath, JSON.stringify({ assets: genesisAssets }, null, '\t'));
};
exports.writeGenesisAssets = writeGenesisAssets;
//# sourceMappingURL=genesis_block.js.map