"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.startLiskCore = exports.isLiskCoreV3Running = exports.installPM2 = exports.installLiskCore = void 0;
const tslib_1 = require("tslib");
const path_1 = require("path");
const fs = tslib_1.__importStar(require("fs-extra"));
const os_1 = require("os");
const fs_extra_1 = require("fs-extra");
const process_1 = require("./process");
const network_1 = require("./network");
const client_1 = require("../client");
const constants_1 = require("../constants");
const INSTALL_LISK_CORE_COMMAND = 'npm i -g lisk-core@^4.0.0-rc.0';
const INSTALL_PM2_COMMAND = 'npm i -g pm2';
const PM2_FILE_NAME = 'pm2.migrator.config.json';
const START_PM2_COMMAND = `pm2 start ${PM2_FILE_NAME}`;
const DEFAULT_LISK_DATA_DIR = `${(0, os_1.homedir)()}/.lisk/lisk-core`;
const LISK_V3_BACKUP_DATA_DIR = `${(0, os_1.homedir)()}/.lisk/lisk-core-v3`;
const installLiskCore = async () => (0, process_1.execAsync)(INSTALL_LISK_CORE_COMMAND);
exports.installLiskCore = installLiskCore;
const installPM2 = async () => (0, process_1.execAsync)(INSTALL_PM2_COMMAND);
exports.installPM2 = installPM2;
const isLiskCoreV3Running = async (liskCorePath) => {
    try {
        const client = await (0, client_1.getAPIClient)(liskCorePath);
        const nodeInfo = await client.node.getNodeInfo();
        return !!nodeInfo;
    }
    catch (_) {
        return false;
    }
};
exports.isLiskCoreV3Running = isLiskCoreV3Running;
const backupDefaultDirectoryIfExists = async (_this) => {
    if ((0, fs_extra_1.existsSync)(DEFAULT_LISK_DATA_DIR)) {
        _this.log(`Backing Lisk Core v3 data directory at ${DEFAULT_LISK_DATA_DIR}`);
        (0, fs_extra_1.renameSync)(DEFAULT_LISK_DATA_DIR, LISK_V3_BACKUP_DATA_DIR);
        _this.log(`Backed Lisk Core v3 data directory to: ${LISK_V3_BACKUP_DATA_DIR}`);
    }
};
const startLiskCore = async (_this, liskCoreV3DataPath, _config, network, outputDir) => {
    var _a, _b, _c, _d;
    const isCoreV3Running = await (0, exports.isLiskCoreV3Running)(liskCoreV3DataPath);
    if (isCoreV3Running)
        throw new Error('Lisk Core v3 is still running.');
    const networkPort = (_b = (_a = _config === null || _config === void 0 ? void 0 : _config.network) === null || _a === void 0 ? void 0 : _a.port) !== null && _b !== void 0 ? _b : constants_1.DEFAULT_PORT_P2P;
    if (!(await (0, network_1.isPortAvailable)(networkPort))) {
        throw new Error(`Port ${networkPort} is not available for P2P communication.`);
    }
    const rpcPort = (_d = (_c = _config === null || _config === void 0 ? void 0 : _config.network) === null || _c === void 0 ? void 0 : _c.port) !== null && _d !== void 0 ? _d : constants_1.DEFAULT_PORT_RPC;
    if (!(await (0, network_1.isPortAvailable)(rpcPort))) {
        throw new Error(`Port ${rpcPort} is not available to start the RPC server.`);
    }
    await backupDefaultDirectoryIfExists(_this);
    _this.log('Installing pm2...');
    await (0, exports.installPM2)();
    _this.log('Finished installing pm2.');
    const customConfigFilepath = (0, path_1.resolve)(outputDir, 'custom_config.json');
    fs.writeFileSync(customConfigFilepath, JSON.stringify({
        ..._config,
        genesis: {
            ..._config.genesis,
            block: {
                fromFile: `${outputDir}/genesis_block.blob`,
            },
        },
    }, null, '\t'));
    fs.writeFileSync(PM2_FILE_NAME, JSON.stringify({
        name: 'lisk-core',
        script: `lisk-core start --network ${network} --config ${customConfigFilepath}`,
    }, null, '\t'));
    return (0, process_1.execAsync)(START_PM2_COMMAND);
};
exports.startLiskCore = startLiskCore;
//# sourceMappingURL=node.js.map