"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTransactions = exports.keyString = void 0;
const lisk_cryptography_1 = require("@liskhq/lisk-cryptography");
const lisk_codec_1 = require("@liskhq/lisk-codec");
const lisk_chain_1 = require("@liskhq/lisk-chain");
const constants_1 = require("../constants");
const keyString = (key) => key.toString('binary');
exports.keyString = keyString;
const getTransactions = async (blockID, db) => {
    try {
        const txIDs = [];
        const ids = await db.get(Buffer.from(`${constants_1.DB_KEY_TRANSACTIONS_BLOCK_ID}:${(0, exports.keyString)(blockID)}`));
        for (let idx = 0; idx < ids.length; idx += constants_1.TRANSACTION_ID_LENGTH) {
            txIDs.push(ids.slice(idx, idx + constants_1.TRANSACTION_ID_LENGTH));
        }
        if (txIDs.length === 0) {
            return [];
        }
        const transactions = [];
        for (const txID of txIDs) {
            const tx = await db.get(Buffer.from(`${constants_1.DB_KEY_TRANSACTIONS_ID}:${(0, exports.keyString)(txID)}`));
            const decodedTransactions = (await lisk_codec_1.codec.decode(lisk_chain_1.transactionSchema, tx));
            const id = (0, lisk_cryptography_1.hash)(tx);
            transactions.push({ ...decodedTransactions, id });
        }
        return transactions;
    }
    catch (error) {
        return [];
    }
};
exports.getTransactions = getTransactions;
//# sourceMappingURL=transaction.js.map